"use strict";

exports.__esModule = true;
exports.useActiveDescendant = exports.setActiveDescendant = void 0;

var _react = require("react");

var _reactDom = require("react-dom");

/**
 * Do a "hard" set on the aria, so that it's always announced
 * even if the id hasn't changed, this saves us from having to have a different id
 * per item.
 */
var setActiveDescendant = function setActiveDescendant(ref, activeId, visible) {
  if (!ref) return;
  var node = (0, _reactDom.findDOMNode)(ref);
  node.removeAttribute('aria-activedescendant');
  node.setAttribute('aria-activedescendant', visible ? activeId : '');
};

exports.setActiveDescendant = setActiveDescendant;

var useActiveDescendant = function useActiveDescendant(ref, id, visible, deps) {
  (0, _react.useEffect)(function () {
    setActiveDescendant(ref.current, id, visible);
  }, [visible].concat(deps));
};

exports.useActiveDescendant = useActiveDescendant;