"use strict";

exports.__esModule = true;
exports.default = useTabTrap;

var _react = require("react");

var selector = ['input', 'button:not([tabindex="-1"])', '[tabindex="0"]'].join(',');

function useTabTrap(ref) {
  var startedRef = (0, _react.useRef)(false);

  function keyHandler(event) {
    if (!ref.current || event.key !== 'Tab') return;
    var tabbables = ref.current.querySelectorAll(selector);

    if (event.shiftKey && event.target === tabbables[0]) {
      tabbables[tabbables.length - 1].focus();
      event.preventDefault();
    } else if (!event.shiftKey && event.target === tabbables[tabbables.length - 1] || !ref.current.contains(event.target)) {
      tabbables[0].focus();
      event.preventDefault();
    }
  }

  (0, _react.useEffect)(function () {
    return function () {
      document.removeEventListener('keydown', keyHandler);
    };
  }, [startedRef]);
  return {
    focus: function focus() {
      var _ref$current, _tabbables$;

      var tabbables = (_ref$current = ref.current) === null || _ref$current === void 0 ? void 0 : _ref$current.querySelectorAll(selector);
      (_tabbables$ = tabbables[0]) === null || _tabbables$ === void 0 ? void 0 : _tabbables$.focus();
    },
    start: function start() {
      if (!startedRef.current) {
        startedRef.current = true;
        document.addEventListener('keydown', keyHandler);
      }
    },
    stop: function stop() {
      startedRef.current = false;
      document.removeEventListener('keydown', keyHandler);
    }
  };
}

module.exports = exports["default"];